﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mediachase.Commerce.Catalog.Objects;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Catalog.Managers;
using System.Web.UI.HtmlControls;
using System.Collections;
using Mediachase.Commerce.Website.Helpers;
using Mediachase.Commerce.Engine;

namespace Mediachase.Commerce.Website.BaseControls
{
    public class BaseEntryTemplate : BaseStoreUserControl, IContextUserControl
    {
        #region Private Variables
        string _CatalogName;
        private Entry _CurrentEntry = null;
        string _Code;
        #endregion

        #region Properties
        /// <summary>
        /// Gets the name of the catalog.
        /// </summary>
        /// <value>The name of the catalog.</value>
        public virtual string CatalogName
        {
            get
            {
                return _CatalogName;
            }
        }

        /// <summary>
        /// Gets the code.
        /// </summary>
        /// <value>The code.</value>
        public virtual string Code
        {
            get
            {
                return _Code;
            }
        }

        /// <summary>
        /// Gets the entry.
        /// </summary>
        /// <value>The entry.</value>
        public virtual Entry Entry
        {
            get
            {
                if (_CurrentEntry == null)
                {
                    _CurrentEntry = LoadNewEntry();
                }

                return _CurrentEntry;
            }
        }

        private Currency _Currency;

        /// <summary>
        /// Gets or sets the currency.
        /// </summary>
        /// <value>The currency.</value>
        public virtual Currency Currency
        {
            get { return _Currency; }
            set { _Currency = value; }
        }

        /// <summary>
        /// Gets or sets the currency code.
        /// </summary>
        /// <value>The currency code.</value>
        [Obsolete("Use the Currency member instead.")]
        public virtual string CurrencyCode
        {
            get { return _Currency.CurrencyCode; }
            set { _Currency = new Currency(value); }
        }

        private string _ShoppingCartUrl;

        /// <summary>
        /// Gets or sets the shopping cart URL.
        /// </summary>
        /// <value>The shopping cart URL.</value>
        public virtual string ShoppingCartUrl
        {
            get { return _ShoppingCartUrl; }
            set { _ShoppingCartUrl = value; }
        }

        private Guid _SiteId = Guid.Empty;

        /// <summary>
        /// Gets or sets the site id.
        /// </summary>
        /// <value>The site id.</value>
        public virtual Guid SiteId
        {
            get { return _SiteId; }
            set { _SiteId = value; }
        }

        private string _WishListUrl;

        /// <summary>
        /// Gets or sets the wish list URL.
        /// </summary>
        /// <value>The wish list URL.</value>
        public virtual string WishListUrl
        {
            get { return _WishListUrl; }
            set { _WishListUrl = value; }
        }

        #endregion

        /// <summary>
        /// Loads the new entry.
        /// </summary>
        /// <returns></returns>
        protected virtual Entry LoadNewEntry()
        {
            return CatalogContext.Current.GetCatalogEntry(Code, new CatalogEntryResponseGroup(CatalogEntryResponseGroup.ResponseGroup.CatalogEntryFull));
        }

        /// <summary>
        /// Loads the context. Implements interface IContextUserControl.
        /// </summary>
        /// <param name="context">The context.</param>
        public virtual void LoadContext(IDictionary context)
        {
            if (context["Code"] != null)
                _Code = context["Code"].ToString();

            if (context["CatalogName"] != null)
                _CatalogName = context["CatalogName"].ToString();

            if (context.Contains("ShoppingCartUrl"))
                this.ShoppingCartUrl = (string)context["ShoppingCartUrl"];

            if (context.Contains("WishListUrl"))
                this.WishListUrl = (string)context["WishListUrl"];

            if (context.Contains("SiteId"))
                this.SiteId = (Guid)context["SiteId"];

            if (context.Contains("CurrencyCode"))
                this.Currency = new Currency((string)context["CurrencyCode"]);
        }

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.PreRender"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            RenderHeaders();
            base.OnPreRender(e);
        }

        /// <summary>
        /// Sets the headers.
        /// </summary>
        protected virtual void RenderHeaders()
        {
            if (Entry == null)
                return;

            Seo seo = StoreHelper.GetLanguageSeo(Entry.SeoInfo);

            if (seo == null && Entry.ItemAttributes["DisplayName"] != null && Entry.ItemAttributes["DisplayName"].Value != null && Entry.ItemAttributes["DisplayName"].Value.Length > 0)
            {
                this.Page.Title = Entry.ItemAttributes["DisplayName"].Value[0];
            }
            else
            {
                this.Page.Title = seo.Title;

				if (String.IsNullOrEmpty(seo.Title))
				{
					if (!String.IsNullOrEmpty(Entry.ItemAttributes["DisplayName"].ToString()))
						this.Page.Title = Entry.ItemAttributes["DisplayName"].ToString();
					else
						this.Page.Title = Entry.Name;
				}

                HtmlMeta ctrl = Page.FindControl("MetaKeyWord") as HtmlMeta;
                if (ctrl != null)
                {
                    ctrl.Attributes.Add("content", seo.Keywords);
                    ctrl.Attributes.Add("name", "keywords");
                }

                ctrl = Page.FindControl("MetaDescription") as HtmlMeta;
                if (ctrl != null)
                {
                    ctrl.Attributes.Add("content", seo.Description);
                    ctrl.Attributes.Add("name", "description");
                }
            }
        }
    }
}
