using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Marketing;
using Mediachase.Commerce.Marketing.Objects;
using System.Collections.Generic;
using Mediachase.Commerce.Catalog;
using Mediachase.Commerce.Catalog.Dto;
using Mediachase.Commerce.Customers.Profile;
using System.Collections.Specialized;
using Mediachase.Commerce.Catalog.Managers;
using System.Web.Security;
using System.Linq;
using Mediachase.MetaDataPlus;


namespace Mediachase.Commerce.Workflow.Activities.Cart
{
	public partial class AdjustInventoryActivity : CartActivityBase
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="AdjustInventoryActivity"/> class.
        /// </summary>
        public AdjustInventoryActivity()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called by the workflow runtime to execute an activity.
        /// </summary>
        /// <param name="executionContext">The <see cref="T:System.Workflow.ComponentModel.ActivityExecutionContext"/> to associate with this <see cref="T:System.Workflow.ComponentModel.Activity"/> and execution.</param>
        /// <returns>
        /// The <see cref="T:System.Workflow.ComponentModel.ActivityExecutionStatus"/> of the run task, which determines whether the activity remains in the executing state, or transitions to the closed state.
        /// </returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            try
            {
                // Validate the properties at runtime
                this.ValidateRuntime();
				var orderForm = OrderGroup.OrderForms[0];
				var orderLineItems = orderForm.LineItems.ToArray().Concat(orderForm.LineItems.DeletedLineItems);
				foreach (var lineItem in orderLineItems)
				{
					AdjustStockItemQuantity(lineItem);
				}

                // Retun the closed status indicating that this activity is complete.
                return ActivityExecutionStatus.Closed;
            }
            catch
            {
                // An unhandled exception occured.  Throw it back to the WorkflowRuntime.
                throw;
            }
        }

		private static void AdjustStockItemQuantity(LineItem lineItem)
		{
			if (lineItem.CatalogEntryId == "0" && 
				String.IsNullOrEmpty(lineItem.CatalogEntryId) &&
				lineItem.CatalogEntryId.StartsWith("@"))
			{
				return;
			}
			var entryDto = CatalogContext.Current.GetCatalogEntryDto(lineItem.CatalogEntryId, 
																	new CatalogEntryResponseGroup(CatalogEntryResponseGroup.ResponseGroup.CatalogEntryFull));

			var catalogEntry = GetCatalogEntry(entryDto);
			if (catalogEntry != null && CheckNeedEntryTracking(catalogEntry))
			{
				var inventory = catalogEntry.InventoryRow;
				if (inventory != null)
				{
					AdjustStockInventoryQuantity(lineItem, inventory);
					// Save entry
					CatalogContext.Current.SaveCatalogEntry(entryDto);
				}
			}
		}

        /// <summary>
        /// Adjusts the stock inventory quantities. Method assumes only inventory TRACKING is enabled.
        /// </summary>
        /// <param name="lineItem">Line item object in cart.</param>
        /// <param name="inventory">Inventory associated with the line item's catalog entry.</param>
		private static void AdjustStockInventoryQuantity(LineItem lineItem, CatalogEntryDto.InventoryRow inventory)
		{
			var delta = GetLineItemAdjustedQuantity(lineItem);

			//arrival
			if (delta > 0)
			{
				// need distribute delta between InStock, Backorder, Preorder.
				if (lineItem.InStockQuantity > 0)
				{
					var backorderDelta = Math.Min(delta, lineItem.BackorderQuantity - inventory.BackorderQuantity);
					var preorderdelta = Math.Min(delta, lineItem.PreorderQuantity - inventory.PreorderQuantity);
					inventory.PreorderQuantity += preorderdelta;
					inventory.BackorderQuantity += backorderDelta;
					inventory.InStockQuantity += delta - backorderDelta - preorderdelta;
				} //need distribute delta between Preorder and Backorder
				else if (lineItem.InStockQuantity == 0)
				{
					if (lineItem.PreorderQuantity > 0)
					{
						inventory.PreorderQuantity += delta;
					}
					else if (lineItem.BackorderQuantity > 0)
					{
						inventory.BackorderQuantity += delta;
					}
				}
			}//consumption
			else
			{
				delta = Math.Abs(delta);
                bool inventoryEnabled = inventory.InventoryStatus == InventoryStatus.Enabled;
                if (inventory.InStockQuantity >= delta) // Adjust the main inventory
				{
                    if (inventoryEnabled)
                    {
                        inventory.InStockQuantity -= (delta - inventory.ReservedQuantity);
                        inventory.BackorderQuantity -= inventory.ReservedQuantity; 
                    }
                    else
                    {
                        inventory.InStockQuantity -= delta;
                    }
				}
				else if (inventory.InStockQuantity > 0) // there still exist items in stock
				{
					// Calculate difference between currently availbe and backorder
					var backorderDelta = delta - (inventory.InStockQuantity - (inventoryEnabled ? inventory.ReservedQuantity : 0));

					// Update inventory
                    if (inventoryEnabled)
                    {
                        inventory.InStockQuantity = inventory.ReservedQuantity;
                        inventory.BackorderQuantity -= backorderDelta; 
                    }
                    else
                    {
                        inventory.InStockQuantity -= delta;
                    }
				}
				else if (inventory.InStockQuantity == 0)
				{
					// Update inventory
					inventory.InStockQuantity = -delta;

                    if (inventoryEnabled)
                    {
                        if (inventory.PreorderQuantity == 0)
                            inventory.BackorderQuantity -= delta;
                        else
                            inventory.PreorderQuantity -= delta; 
                    }
				}

			}
		}

		private static decimal GetLineItemAdjustedQuantity(LineItem lineItem)
		{

			decimal retVal = 0m;
			if (lineItem.Parent.Parent is Mediachase.Commerce.Orders.Cart)
			{
				retVal = -lineItem.Quantity;
			}
			else if (lineItem.ObjectState == MetaObjectState.Added)
			{
				retVal = -lineItem.Quantity;
			}
			else if (lineItem.ObjectState == MetaObjectState.Deleted)
			{
				retVal = lineItem.Quantity;
			}
			else if (lineItem.ObjectState == MetaObjectState.Modified)
			{
				retVal = lineItem.OldQuantity - lineItem.Quantity;
			}
			
			return retVal;
		}

		private static bool CheckNeedEntryTracking(CatalogEntryDto.CatalogEntryRow catalogEntry)
		{
			if (catalogEntry == null)
			{
				throw new ArgumentNullException("catalogEntry");
			}
			var retVal = false;
			var entryVariations = GetEntryVariations(catalogEntry);
			var variation = entryVariations.FirstOrDefault();
			if (variation != null)
			{
				retVal = variation.TrackInventory;
			}
			return retVal;
		}

		private static CatalogEntryDto.CatalogEntryRow GetCatalogEntry(CatalogEntryDto entryDto)
		{
			if (entryDto == null)
			{
				throw new ArgumentNullException("entryDto");
			}
			return entryDto.CatalogEntry.FirstOrDefault();
		}

		private static IEnumerable<CatalogEntryDto.VariationRow> GetEntryVariations(CatalogEntryDto.CatalogEntryRow catalogEntry)
		{
			if (catalogEntry == null)
			{
				throw new ArgumentNullException("catalogEntry");
			}
			return catalogEntry.GetVariationRows();
		}
    }
}
