﻿using System;
using System.Data;
using Mediachase.Commerce.Orders.Dto;
using Mediachase.Web.Console.Interfaces;

namespace EPiServer.Business.Commerce.Payment.DataCash
{
    public partial class ConfigurePayment : System.Web.UI.UserControl, IGatewayControl
    {
        // Fields
        private PaymentMethodDto _paymentMethodDto;
        private string _validationGroup;

        /// <summary>
        /// Initializes a new instance of the <see cref="ConfigurePayment"/> class.
        /// </summary>
        public ConfigurePayment()
        {
            _validationGroup = string.Empty;
            _paymentMethodDto = null;
        }

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            BindData();
        }

        /// <summary>
        /// Binds the data.
        /// </summary>
        public void BindData()
        {
            if ((_paymentMethodDto != null) && (_paymentMethodDto.PaymentMethodParameter != null))
            {
                HostAddress.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.HostAddressParameter);
                APIUser.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.UserIdParameter);
                Password.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.PasswordParameter);
                LogFilePath.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.PathToLogFileParameter);
                TimeOut.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.TimeOutParameter);
                Proxy.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.ProxyParameter);
                PaymentPageId.Text = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.PaymentPageIdParameter);

                var selectedValue = _paymentMethodDto.GetParameterValueAsString(DataCashPaymentGateway.LoggingLevelParameter);
                LoggingLevel.SelectedValue = string.IsNullOrEmpty(selectedValue) ? "5" : selectedValue;
            }
            else
            {
                Visible = false;
            }
        }

        #region IGatewayControl Members

        /// <summary>
        /// Loads the object.
        /// </summary>
        /// <param name="dto">The dto.</param>
        public void LoadObject(object dto)
        {
            _paymentMethodDto = dto as PaymentMethodDto;
        }

        /// <summary>
        /// Saves the changes.
        /// </summary>
        /// <param name="dto">The dto.</param>
        public void SaveChanges(object dto)
        {
            if (Visible)
            {
                _paymentMethodDto = dto as PaymentMethodDto;
                if (_paymentMethodDto != null && _paymentMethodDto.PaymentMethodParameter != null)
                {
                    Guid paymentMethodId = Guid.Empty;
                    if (_paymentMethodDto.PaymentMethod.Count > 0)
                    {
                        paymentMethodId = _paymentMethodDto.PaymentMethod[0].PaymentMethodId;
                    }
                    UpdateParameter(DataCashPaymentGateway.HostAddressParameter, HostAddress.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.UserIdParameter, APIUser.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.PasswordParameter, Password.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.PathToLogFileParameter, LogFilePath.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.TimeOutParameter, TimeOut.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.LoggingLevelParameter, LoggingLevel.SelectedValue, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.ProxyParameter, Proxy.Text, paymentMethodId);
                    UpdateParameter(DataCashPaymentGateway.PaymentPageIdParameter, string.IsNullOrEmpty(PaymentPageId.Text) ? "1" : PaymentPageId.Text, paymentMethodId);
                }
            }
        }

        /// <summary>
        /// Updates the parameter.
        /// </summary>
        /// <param name="paramName">Name of the param.</param>
        /// <param name="value">The value.</param>
        /// <param name="paymentMethodId">The payment method id.</param>
        private void UpdateParameter(string paramName, string value, Guid paymentMethodId)
        {
            PaymentMethodDto.PaymentMethodParameterRow parameterByName = _paymentMethodDto.GetParameterByName(paramName);
            if (parameterByName != null)
            {
                parameterByName.Value = value;
            }
            else
            {
                CreateParameter(_paymentMethodDto, paramName, value, paymentMethodId);
            }
        }

        /// <summary>
        /// Creates the parameter.
        /// </summary>
        /// <param name="dto">The dto.</param>
        /// <param name="name">The name.</param>
        /// <param name="value">The value.</param>
        /// <param name="paymentMethodId">The payment method id.</param>
        private void CreateParameter(PaymentMethodDto dto, string name, string value, Guid paymentMethodId)
        {
            PaymentMethodDto.PaymentMethodParameterRow row = dto.PaymentMethodParameter.NewPaymentMethodParameterRow();
            row.PaymentMethodId = paymentMethodId;
            row.Parameter = name;
            row.Value = value;
            if (row.RowState == DataRowState.Detached)
            {
                dto.PaymentMethodParameter.Rows.Add(row);
            }
        }

        /// <summary>
        /// Gets or sets the validation group.
        /// </summary>
        /// <value>The validation group.</value>
        public string ValidationGroup
        {
            get
            {
                return _validationGroup;
            }
            set
            {
                _validationGroup = value;
            }
        }

        #endregion
    }
}