﻿using System.Web;
using System.Web.Routing;
using Mediachase.Commerce.Catalog.Dto;
using Mediachase.Commerce.Catalog;

namespace Mediachase.Commerce.Website.Handlers
{
    /// <summary>
    /// The CatalogEntry route class supports adding catalog entry code and catalog entry entity to current route data of request.
    /// </summary>
    public class CatalogEntryRoute : Route
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="CatalogEntryRoute"/> class.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <param name="physicalFile">The physical file.</param>
        public CatalogEntryRoute(string url, string physicalFile)
            : base(url, new PageRouteHandler(physicalFile, false))
        {
        }

        /// <summary>
        /// Try to inspect the URL, returns route data containing catalog entry code and catalog entry entity route data.
        /// </summary>
        /// <param name="httpContext">An object that encapsulates information about the HTTP request.</param>
        /// <returns>
        /// An <see cref="System.Web.Routing.RouteData"/> object that contains values for "ec" and "EntryDto"
        /// </returns>
        public override RouteData GetRouteData(HttpContextBase httpContext)
        {
            // Try to get CatalogEntry matched the friendly url first
            string remainingPath = httpContext.Request.AppRelativeCurrentExecutionFilePath.Substring(2) + httpContext.Request.PathInfo;
            CatalogEntryDto entry = CatalogContext.Current.GetCatalogEntryByUriDto(remainingPath, null);
            var routeData = new RouteData(this, RouteHandler); ;
            if (entry.CatalogItemSeo.Count == 0)
            {
                // Get matched CatalogEntry from entry code in query string
                var entryCode = httpContext.Request.QueryString["ec"];
                if (!string.IsNullOrEmpty(entryCode))
                {
                    entry = CatalogContext.Current.GetCatalogEntryDto(entryCode);
                }
            }

            // Cannot find any matched catalog entry
            if (entry == null || entry.CatalogEntry.Count == 0)
            {
                return null;
            }

            routeData.DataTokens.Add("ec", entry.CatalogEntry[0].Code);
            routeData.DataTokens.Add("EntryDto", entry);
            return routeData;
        }
    }
}
