using System;
using System.Collections.Generic;
using System.Text;
using Mediachase.Commerce.Marketing.Objects;
using Mediachase.Commerce.Marketing;
using Mediachase.Commerce.Catalog.Objects;
using Mediachase.Commerce.Catalog.Managers;
using Mediachase.Commerce.Orders;
using Mediachase.Commerce.Customers.Profile;
using System.Web.Security;
using Mediachase.Commerce.Security;
using Mediachase.Commerce.Customers;

namespace Mediachase.Commerce.Website.Helpers
{
    public class PromotionHelper
    {
        PromotionContext _Context = null;

        /// <summary>
        /// Gets or sets the promotion context.
        /// </summary>
        /// <value>The promotion context.</value>
        public PromotionContext PromotionContext
        {
            get { return _Context; }
            set { _Context = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PromotionHelper"/> class.
        /// </summary>
        /// <param name="context">The context.</param>
        public PromotionHelper(PromotionContext context)
        {
            InitPromotionContext();
            _Context = context;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PromotionHelper"/> class.
        /// </summary>
        public PromotionHelper()
        {
            InitPromotionContext();
            _Context = new PromotionContext(MarketingContext.Current.MarketingProfileContext, null, null);
        }

        /// <summary>
        /// Evals the specified filter.
        /// </summary>
        /// <param name="filter">The filter.</param>
        public void Eval(PromotionFilter filter)
        {
            MarketingContext.Current.EvaluatePromotions(true, this.PromotionContext, filter, null, false);
        }

        /// <summary>
        /// Inits the promotion context.
        /// </summary>
        public void InitPromotionContext()
        {
            // We assume if there is already shopping cart context defined for the current http request, that we do not need to reinitialize it again
            if (!MarketingContext.Current.MarketingProfileContext.ContainsKey(MarketingContext.ContextConstants.ShoppingCart))
            {
                SetContext(MarketingContext.ContextConstants.ShoppingCart, OrderContext.Current.GetCart(Cart.DefaultName, SecurityContext.Current.CurrentContactId));
                //SetContext(MarketingContext.ContextConstants.CustomerProfile, ProfileContext.Current.Profile);

                // Set customer segment context
                MembershipUser user = SecurityContext.Current.CurrentUser;
                if (user != null)
                {
					CustomerContact cutomerContact = CustomerContext.Current.GetContactForUser(SecurityContext.Current.CurrentUser);
					if (cutomerContact != null)
                    {
						SetContext(MarketingContext.ContextConstants.CustomerContact, cutomerContact);
						Guid contactId = (Guid)cutomerContact.PrimaryKeyId;
                        Guid organizationId = Guid.Empty;
						if (cutomerContact.ContactOrganization != null)
                        {
							organizationId = (Guid)cutomerContact.ContactOrganization.PrimaryKeyId;
                        }

						SetContext(MarketingContext.ContextConstants.CustomerSegments, MarketingContext.Current.GetCustomerSegments(contactId, organizationId));
                    }
                }

                // Set customer promotion history context
                SetContext(MarketingContext.ContextConstants.CustomerId, SecurityContext.Current.CurrentContactId);
            }
        }         

        /// <summary>
        /// Sets the context.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="val">The val.</param>
        private void SetContext(string key, object val)
        {
            if (!MarketingContext.Current.MarketingProfileContext.ContainsKey(key))
                MarketingContext.Current.MarketingProfileContext.Add(key, val);
            else
                MarketingContext.Current.MarketingProfileContext[key] = val;
        }
    }
}
