﻿using System;
using System.Text.RegularExpressions;
using log4net;

namespace EPiServer.Business.Commerce.Payment.PayPal
{
    /// <summary>
    /// This page will redirect if RedirectTopWindowUrl has value.
    /// We Render the js script to redirect top window to what we want in this property's value.
    /// We only validate the URL to redirect to Paypal.com sites only (by using Regex).
    /// <remarks>
    /// This is the helper page when we want page (PaymentMethod.aspx) inside iframe (of Checkout.aspx) redirect.
    /// Because when we do business to checkout (PayPal) inside iframe, 
    /// server code in PaymentGateway cannot tell the browser to redirect because HttpContext.Response.Redirect will make the iframe redirect (to Paypal) and it's prohibited by security (Paypal.com site does not rendered inside iframe).
    /// </remarks>
    /// </summary>
    public partial class PayPalRedirect : System.Web.UI.Page
    {
        private static ILog _logger = LogManager.GetLogger(typeof(PayPal));

        /// <summary>
        /// If this property have value, it means this page should work in Redirect mode
        /// </summary>
        protected string RedirectTopWindowUrl { get; set; }
        
        /// <summary>
        /// Match if url start with https, belong to *.paypal.com
        /// </summary>
        private Regex regexPaypalUrl = new Regex("^https://[^/]+paypal.com/.+$", RegexOptions.IgnoreCase | RegexOptions.Compiled | RegexOptions.Singleline);

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            // use js to make the parent (Checkout.aspx) change location to PayPal url
            string sRedirectUrl = Request["RedirectUrl"] as string;
            if (string.IsNullOrEmpty(sRedirectUrl) || !regexPaypalUrl.IsMatch(sRedirectUrl))
            {
                // do nothing (to prevent bad abuse) because the url is null, or not belong to paypal.com's sites
                return;
            }

            // url is fine, tell the browser window to redirect
            RedirectTopWindowUrl = sRedirectUrl;
            ClientScriptToRedirectTopWindow.Visible = true;
        }
    }
}
