﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mediachase.Commerce.Engine.Navigation;
using System.Collections.Specialized;

namespace Mediachase.Commerce.Website
{
    public class StaticUrlProvider : UrlProvider
    {
        private StringDictionary _Urls = null;

        /// <summary>
        /// Initializes the provider.
        /// </summary>
        /// <param name="name">The friendly name of the provider.</param>
        /// <param name="config">A collection of the name/value pairs representing the provider-specific attributes specified in the configuration for this provider.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// The name of the provider is null.
        /// </exception>
        /// <exception cref="T:System.ArgumentException">
        /// The name of the provider has a length of zero.
        /// </exception>
        /// <exception cref="T:System.InvalidOperationException">
        /// An attempt is made to call <see cref="M:System.Configuration.Provider.ProviderBase.Initialize(System.String,System.Collections.Specialized.NameValueCollection)"/> on a provider after the provider has already been initialized.
        /// </exception>
        public override void Initialize(string name, NameValueCollection config)
        {
            base.Initialize(name, config);

            _Urls = new StringDictionary();
            _Urls.Add("Checkout", "~/Checkout.aspx");
            _Urls.Add("ShoppingCart", "~/ShoppingCart.aspx");
            _Urls.Add("WishList", "~/WishList.aspx");
            _Urls.Add("EntryView", "~/Handlers/EntryHandler.aspx");
            _Urls.Add("NodeView", "~/Handlers/NodeHandler.aspx");
            _Urls.Add("Register", "~/Register.aspx");
            _Urls.Add("RecoverPassword", "~/RecoverPassword.aspx");
            _Urls.Add("OrderHistory", "~/Profile/OrdersList.aspx");
            _Urls.Add("Account", "~/Profile/Dashboard.aspx");
            _Urls.Add("Compare", "~/ProductsCompare.aspx");
            _Urls.Add("SearchResults", "~/SearchResults.aspx");
        }

        /// <summary>
        /// Gets the URL.
        /// </summary>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="commandParams">The command params.</param>
        /// <returns></returns>
        public override string GetUrl(string commandName, params object[] commandParams)
        {
            string url = GetUrl(commandName);

            // now add parameters
            for (int index = 0; index < commandParams.Length; index += 2)
            {
                if(!url.Contains("?"))
                    url += "?";
                else
                    url += "&";

                url += String.Format("{0}={1}", commandParams[index], commandParams[index+1]);
            }

            return url;
        }

        /// <summary>
        /// Gets the URL.
        /// </summary>
        /// <param name="commandName">Name of the command.</param>
        /// <returns></returns>
        public override string GetUrl(string commandName)
        {
            return _Urls[commandName];
        }
    }
}
